@@ -18,6 +18,7 @@ SEXP do_rprior (SEXP object, SEXP params, SEXP gnsi)
   int mode = -1;
   int npars, nreps;
   SEXP Pnames, P, fn, fcall;
+  SEXP pompfun;
   int *dim;
 
   PROTECT(params = as_matrix(params)); nprotect++;
@@ -27,7 +28,8 @@ SEXP do_rprior (SEXP object, SEXP params, SEXP gnsi)
   PROTECT(Pnames = GET_ROWNAMES(GET_DIMNAMES(params))); nprotect++;
     
   // extract the user-defined function
-  PROTECT(fn = pomp_fun_handler(GET_SLOT(object,install("rprior")),gnsi,&mode)); nprotect++;
+  PROTECT(pompfun = GET_SLOT(object,install("rprior"))); nprotect++;
+  PROTECT(fn = pomp_fun_handler(pompfun,gnsi,&mode)); nprotect++;
 
   // extract 'userdata' as pairlist
   PROTECT(fcall = VectorToPairList(GET_SLOT(object,install("userdata")))); nprotect++;
@@ -78,7 +80,7 @@ SEXP do_rprior (SEXP object, SEXP params, SEXP gnsi)
 	  PROTECT(nm = GET_NAMES(ans)); nprotect++;
 	  use_names = !isNull(nm);
 	  if (use_names) {   // match names against names from params slot
-	    posn = INTEGER(PROTECT(matchnames(Pnames,nm))); nprotect++;
+	    posn = INTEGER(PROTECT(matchnames(Pnames,nm,"parameters"))); nprotect++;
 	  } else {
 	    posn = 0;
 	  }
@@ -114,7 +116,7 @@ SEXP do_rprior (SEXP object, SEXP params, SEXP gnsi)
       PROTECT(P = duplicate(params)); nprotect++;
 
       // construct state, parameter, covariate, observable indices
-      pidx = INTEGER(PROTECT(name_index(Pnames,object,"paramnames"))); nprotect++;
+      pidx = INTEGER(PROTECT(name_index(Pnames,pompfun,"paramnames"))); nprotect++;
       
       // address of native routine
       ff = (pomp_rprior *) R_ExternalPtrAddr(fn);
